@@ -1,21 +1,43 @@
 from collections import OrderedDict
 
-from etc.toolbox import get_model_class_from_settings
+from etc.toolbox import get_model_class_from_string
+from django import VERSION
 from django.contrib.contenttypes.models import ContentType
 from django.core.cache import cache
 from django.db.models import signals, Count, Model
 
-from sitecats import settings
+from .settings import MODEL_CATEGORY, MODEL_TIE
 
 
 def get_category_model():
     """Returns the Category model, set for the project."""
-    return get_model_class_from_settings(settings, 'MODEL_CATEGORY')
+    return get_model_class_from_string(MODEL_CATEGORY)
 
 
 def get_tie_model():
     """Returns the Tie model, set for the project."""
-    return get_model_class_from_settings(settings, 'MODEL_TIE')
+    return get_model_class_from_string(MODEL_TIE)
+
+
+_SITECATS_CACHE = None
+
+
+def get_cache():
+    """Returns global cache object.
+
+    :rtype: Cache
+    :return: cache object
+    """
+    global _SITECATS_CACHE
+
+    if _SITECATS_CACHE is None:
+        if VERSION < (1, 7, 0):
+            _SITECATS_CACHE = Cache()  # Caching object.
+        else:
+            from django.apps import apps
+            _SITECATS_CACHE = apps.get_app_config('sitecats').get_categories_cache()
+
+    return _SITECATS_CACHE
 
 
 class Cache(object):
@@ -168,33 +190,75 @@ class Cache(object):
                 break
         return found
 
-    def get_categories(self, parent_alias=None, target_object=None):
+    def get_ties_stats(self, categories, target_model=None):
+        """Returns a dict with categories popularity stats.
+
+        :param list categories:
+        :param Model|None target_model:
+        :return:
+        """
+        filter_kwargs = {
+            'category_id__in': categories
+        }
+        if target_model is not None:
+            is_cls = hasattr(target_model, '__name__')
+            if is_cls:
+                concrete = False
+            else:
+                concrete = True
+                filter_kwargs['object_id'] = target_model.id
+            filter_kwargs['content_type'] = ContentType.objects.get_for_model(target_model, for_concrete_model=concrete)
+
+        return {item['category_id']: item['ties_num'] for item in get_tie_model().objects.filter(**filter_kwargs).values('category_id').annotate(ties_num=Count('category'))}
+
+    def get_categories(self, parent_aliases=None, target_object=None, tied_only=True):
         """Returns subcategories (or ties if `target_object` is set)
         for the given parent category.
 
-        :param str parent_alias:
+        :param str|None|list parent_aliases:
         :param ModelWithCategory|Model target_object:
+        :param bool tied_only: Flag to get only categories with ties. Ties stats are stored in `ties_num` attrs.
         :return: a list of category objects or tie objects extended with information from their categories.
         """
-        if target_object is None:  # No filtering by object, list all known categories.
-            return self.get_children_for(parent_alias)
-        else:
-            filter_kwargs = {}
-            child_ids = self.get_child_ids(parent_alias)
-            if not child_ids:
-                return []
-            filter_kwargs.update({
-                'content_type': ContentType.objects.get_for_model(target_object),
-                'object_id': target_object.id,
-                'category_id__in': child_ids
-            })
 
-            # Calculating categories weight too.
-            # dicts with `ties_num` and `category_id`
-            stats = {item['category_id']: item['ties_num'] for item in  get_tie_model().objects.filter(**filter_kwargs).values('category_id').annotate(ties_num=Count('category'))}
-            categories = []
-            for cat_id, ties_num in stats.items():
+        single_mode = False
+        if not isinstance(parent_aliases, list):
+            single_mode = parent_aliases
+            parent_aliases = [parent_aliases]
+
+        all_chlidren = []
+        parents_to_children = OrderedDict()
+        for parent_alias in parent_aliases:
+            child_ids = self.get_child_ids(parent_alias)
+            parents_to_children[parent_alias] = child_ids
+            if tied_only:
+                all_chlidren.extend(child_ids)
+
+        ties = {}
+        if tied_only:
+            source = OrderedDict()
+            ties = self.get_ties_stats(all_chlidren, target_object)
+            for parent_alias, child_ids in parents_to_children.items():
+                common = set(ties.keys()).intersection(child_ids)
+                if common:
+                    source[parent_alias] = common
+
+        else:
+            source = parents_to_children
+
+        categories = OrderedDict()
+        for parent_alias, child_ids in source.items():
+            for cat_id in child_ids:
                 cat = self.get_category_by_id(cat_id)
-                cat.ties_num = ties_num
-                categories.append(cat)
-            return categories
+                if tied_only:
+                    cat.ties_num = ties.get(cat_id, 0)
+
+                if parent_alias not in categories:
+                    categories[parent_alias] = []
+
+                categories[parent_alias].append(cat)
+
+        if single_mode != False:  # sic!
+            return categories[single_mode]
+
+        return categories
